{-|
Module      : Gargantext.Core.Viz.Graph.Tools.IGraph
Description : Tools to build Graph
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

Reference:
* Gábor Csárdi, Tamás Nepusz: The igraph software package for complex network research. InterJournal Complex Systems, 1695, 2006.

-}

module Gargantext.Core.Viz.Graph.Tools.IGraph
  ( spinglass
  , mkGraphUfromEdges
  ) where

import Data.Serialize (Serialize)
import IGraph hiding (mkGraph, neighbors, edges, nodes, Node, Graph)
import Protolude
import Gargantext.Prelude (saveAsFileDebug)
import qualified Data.List                   as List
import qualified Data.Map.Strict             as Map
import qualified IGraph                      as IG
import qualified IGraph.Algorithms.Community as IG
import qualified IGraph.Algorithms.Structure as IG
import qualified IGraph.Random               as IG
import qualified Data.Set                    as Set

------------------------------------------------------------------
-- | Partitions

spinglass :: Int -> Map (Int, Int) Double -> IO [Set Int]
spinglass seed graph = map Set.fromList
             <$> List.concat
             <$> mapM (spinglassAux seed) connectedComponents
  where
    -- Non-connected graphs make spinglass crash
    connectedComponents = IG.decompose    -- decompose graph into connected components
                        $ edgeList2UGraph -- convert into IGraph type
                        $ Map.keys graph  -- retrieve edges in the form of `(Int, Int)`


-- | Helper function for `spinglass`
spinglassAux :: (Serialize v, Serialize e, Show v)
                         => Int -> IG.Graph 'U v e -> IO [[v]]
spinglassAux seed graph = do
  gen <- IG.withSeed seed pure
  rawClusters <- IG.findCommunity graph Nothing Nothing IG.spinglass gen
  -- Retrieve node labels from internal node IDs:
  let clusterLabels = (fmap . fmap) (IG.nodeLab graph) rawClusters
  saveAsFileDebug "/tmp/res" clusterLabels
  pure clusterLabels


-- | Make an undirected IGraph graph from a list of edges between `Int`s.
-- The output graph's vertices are labeled with the original `Int`s, and the
-- edges are not labeled.
edgeList2UGraph :: [(Int, Int)] -> IG.Graph 'U Int ()
edgeList2UGraph edgeList =
 -- We're not using `IG.mkGraph` because of the following ticket:
 -- https://gitlab.iscpif.fr/gargantext/haskell-igraph/issues/4
  IG.fromLabeledEdges $ fmap (\edge -> (edge, ())) $ edgeList


-- | Make an "anonymous" (i.e. without labels) graph out of a list of edges.
-- Warning: there is no guarantee, as far as I know, that the underlying
-- representation of the nodes corresponds to the original `Int`.
mkGraphUfromEdges :: [(Int, Int)] -> IG.Graph 'U () ()
mkGraphUfromEdges es = IG.mkGraph (List.replicate n ()) $ zip es $ repeat ()
  where
    (a,b) = List.unzip es
    n = List.length (List.nub $ a <> b)
